/*
 * C
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

#if !defined __MEJ_MATH_H__
# define __MEJ_MATH_H__

// -----------------------------------------------------------------------------
// Includes
// -----------------------------------------------------------------------------

#include "arm_math.h"

// -----------------------------------------------------------------------------
// Macros and Defines
// -----------------------------------------------------------------------------

/*
 * @brief Returns the minimum of two values
 */
# define MEJ_MIN(a, b)  (((a) < (b)) ? (a) : (b))

/*
 * @brief Returns the minimum of two values
 */
# define MEJ_ABS(a)  (((a) < 0) ? (-a) : (a))

/*
 * @brief Returns the minimum of two values
 */
# define MEJ_MAX(a, b)  (((a) < (b)) ? (b) : (a))

/*
 * @brief Computes the percentage between a and b
 */
# define MEJ_PERCENT(a, b) ((100 * (a)) / (b))

/*
 * @brief Converts degrees to radians
 */
# define MEJ_DEG2RAD(d)		(((d) * PI) / 180)

/*
 * @brief Converts radians to degrees
 */
# define MEJ_RAD2DEG(d)		(((d) * 180) / PI)

/*
 * @brief Computes a bounded value
 */
# define MEJ_BOUNDS(v, min, max)		MEJ_MAX((min), MEJ_MIN((v), (max)))

#endif // !defined __MEJ_MATH_H__

/*
 * @brief Floating-point square root function.
 * @param[in] value: input value
 *
 * @return: square root of input value
 */
float32_t mej_sqrt_f32(float32_t value);

/*
 * @brief Floating-point return the tangent of value, where value is given in radians.
 * @param[in] value: input value
 *
 * @return: the tangent of input value
 */
float32_t mej_tan_f32(float32_t value);

// -----------------------------------------------------------------------------
// EOF
// -----------------------------------------------------------------------------

