/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.demo.watch.util.path;

import com.microej.vivante.utils.GradientPath;
import com.verisilicon.vivante.vg_lite_gradient;

import ej.annotation.NonNull;

/**
 * A vector graphics composed of only one gradient path.
 */
public abstract class GradientPathGraphics extends SinglePathGraphics {

	/**
	 * Creates an new vector graphics with a gradient, given the path that composes the vector graphics and the gradient
	 * color information.
	 *
	 * @param path
	 *            the unique path of this vector graphics.
	 * @param gradientStyle
	 *            the gradient style to use for this vector graphics.
	 */
	public GradientPathGraphics(@NonNull GradientPath path, @NonNull GradientStyle gradientStyle) {
		super(path);
		gradientStyle.applyStyle(getGradient());
	}

	/**
	 * Gets the VGLite gradient of the vector image.
	 *
	 * @return the VGLite gradient.
	 */
	public vg_lite_gradient getGradient() {
		return ((GradientPath) getPath()).getGradient();
	}

	/**
	 * Closes this vector graphics and frees any allocated memory used for gradient.
	 */
	@Override
	public void close() {
		super.close();
		getGradient().close();
	}
}
