/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.path.chart;

import com.microej.demo.watch.util.path.GradientPathGraphics;
import com.microej.demo.watch.util.path.GradientStyle;
import com.microej.vivante.utils.GradientPath;
import com.microej.vivante.utils.PathBufferS32;
import com.verisilicon.vivante.vg_lite;
import com.verisilicon.vivante.vg_lite_gradient;
import com.verisilicon.vivante.vg_lite_matrix;
import com.verisilicon.vivante.vg_lite_path;

/**
 * A vector graphics that represents the background of the chart.
 */
public class ChartTimezoneB extends GradientPathGraphics {

	private static final float GRADIENT_SIZE = 256f;

	/**
	 * Creates the background of the chart using a rectangular path and a gradient.
	 *
	 * @param y
	 *            the top y-coordinate of the vector graphics.
	 * @param width
	 *            the width of the vector graphics.
	 * @param height
	 *            the height of the vector graphics.
	 * @param gradientStyle
	 *            the gradient information.
	 *
	 */
	public ChartTimezoneB(int y, int width, int height, GradientStyle gradientStyle) {
		super(createPath(width, height), gradientStyle);
		initialize(y, height, gradientStyle.getAngle());
	}

	private static GradientPath createPath(int width, int height) {
		PathBufferS32 pathBuffer = new PathBufferS32();

		pathBuffer.move(0, 0);
		pathBuffer.line(width, 0);
		pathBuffer.line(width, height);
		pathBuffer.line(0, height);

		vg_lite_path path = pathBuffer.toVGLitePath(0, 0, width, height, vg_lite_path.VG_LITE_HIGH);

		return new GradientPath(path, new vg_lite_gradient(), vg_lite.VG_LITE_FILL_EVEN_ODD);
	}

	private void initialize(int y, int height, int angle) {
		vg_lite_matrix matrix = getMatrix();
		vg_lite_matrix.vg_lite_identity(matrix);
		vg_lite_matrix.vg_lite_translate(0, y, matrix);

		vg_lite_gradient gradient = getGradient();
		assert gradient != null;
		vg_lite_matrix gradientMatrix = gradient.getMatrix();
		vg_lite_matrix.vg_lite_identity(gradientMatrix);
		vg_lite_matrix.vg_lite_translate(0, y, gradientMatrix);
		vg_lite_matrix.vg_lite_rotate(angle, gradientMatrix);
		vg_lite_matrix.vg_lite_scale(height / GRADIENT_SIZE, 1f, gradientMatrix);
	}
}
