/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.classpath.processor;

import java.util.logging.Logger;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

import com.is2t.classpath.ClasspathLoader;
import com.is2t.classpath.EntryFilter;
import com.is2t.classpath.FileClasspath;
import com.is2t.giml.classpath.ClasspathModel;

/**
 * GIML Classpath processor
 */
public class GIMLClasspathProcessor {

	private static final Logger LOGGER = Logger.getLogger(GIMLClasspathProcessor.class.getName());

	private ClasspathModel classpathModel;

	public ClasspathModel process(String[] classpath) {
		classpathModel = new ClasspathModel();
		FileClasspath[] fileClasspath = ClasspathLoader.loadClasspath(classpath);
		for (FileClasspath p : fileClasspath) {
			String[] entries = p.list(new EntryFilter() {

				@Override
				public boolean accept(String arg0) {
					return arg0.endsWith(".class");
				}
			});

			for (String entry : entries) {
				byte[] bytes = p.load(entry);
				processClassfile(p, entry, bytes);
			}
		}
		return classpathModel;
	}

	/**
	 * @param p
	 * @param entry
	 * @param bytes
	 */
	private void processClassfile(FileClasspath p, String entry, byte[] bytes) {
		System.out.println("Processing " + p.toStringFilename(entry));

		ClassReader cr = new ClassReader(bytes);
		ClassVisitor cn = new GIMLClassVisitor();
		cr.accept(cn, ClassReader.SKIP_DEBUG);
	}

	class GIMLClassVisitor extends ClassVisitor {

		GIMLClassVisitor() {
			super(Opcodes.ASM4);
		}

		private String currentType;

		@Override
		public void visit(int version, int access, String name, String signature, String superName,
				String[] interfaces) {
			if ((access & Opcodes.ACC_PUBLIC) != 0) {
				// only
				currentType = name;
			}
			super.visit(version, access, name, signature, superName, interfaces);
		}


		@Override
		public MethodVisitor visitMethod(int access, String name, String desc, String signature, String[] exceptions) {
			if (currentType != null) {
				// consider only public instance methods available in JavaScript world
				if (!("<init>".equals(name)) && (access & Opcodes.ACC_STATIC) == 0
						&& (access & Opcodes.ACC_PUBLIC) != 0) {
					classpathModel.addInstanceMethod(currentType, name, desc);
				}
			}
			return null;
		}

		@Override
		public void visitEnd() {
			super.visitEnd();
			this.currentType = null;
		}
	}
}
