/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.selector;

import java.io.StringWriter;
import java.util.Hashtable;
import java.util.Map;

import org.w3c.css.sac.AttributeCondition;
import org.w3c.css.sac.Condition;

import com.steadystate.css.parser.selectors.IdConditionImpl;

/**
 *
 */
public class IdSelector implements Selector {

	private static final String ID_SELECTOR_VAR_PREFIX = "id_";
	private final Map<String, String> idSelectorVars = new Hashtable<>();

	@Override
	public String generate(Condition c, StringWriter writer) throws IllegalArgumentException {
		AttributeCondition condition = this.get(c);
		String value = condition.getValue();
		String idSelectorVar = this.idSelectorVars.get(value.toLowerCase());
		if (idSelectorVar != null) {
			return idSelectorVar;
		}
		StringBuilder idSelectorVarSb = new StringBuilder(ID_SELECTOR_VAR_PREFIX).append(value);
		writer.append(this.getClassName()).append(" ").append(idSelectorVarSb);
		writer.append(" = new ");
		writer.append(this.getClassName()).append("(\"");
		writer.append(value);
		writer.append("\");\n");
		this.idSelectorVars.put(value.toLowerCase(), idSelectorVarSb.toString());
		return idSelectorVarSb.toString();
	}

	private AttributeCondition get(Condition condition) {
		if (!this.matches(condition)) {
			throw new IllegalArgumentException("Given condition is not an AttributeCondition");
		}
		return (AttributeCondition) condition;
	}

	@Override
	public boolean matches(Condition condition) {
		return condition instanceof IdConditionImpl;
	}

	@Override
	public String getClassName() {
		return "ej.style.selector.IDSelector";
	}

	@Override
	public String generate(org.w3c.css.sac.Selector s, StringWriter writer) throws IllegalArgumentException {
		throw new RuntimeException();
	}

	@Override
	public boolean matches(org.w3c.css.sac.Selector selector) {
		return false;
	}

}
