/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.util;

import java.io.IOException;
import java.io.StringReader;

import org.w3c.css.sac.InputSource;
import org.w3c.css.sac.SelectorList;
import org.w3c.dom.css.CSSRule;

import com.steadystate.css.parser.CSSOMParser;
import com.steadystate.css.parser.SACParserCSS3;

/**
 * The class {@code CSSParserUtils} offers utility methods to manipulate {@link CSSOMParser}.
 */
public class CSSParserUtils {

	private static final CSSOMParser PARSER;

	static {
		PARSER = new CSSOMParser(new SACParserCSS3());
	}

	private CSSParserUtils() {
		// utility class
	}

	/**
	 * Gets the configured instance of the CSS parser. This parser is CSS3 compliant.
	 *
	 * @return the CSS parser.
	 */
	public static CSSOMParser getParser() {
		return PARSER;
	}

	/**
	 * Parse the given CSS selectors.
	 *
	 * @param selectors
	 *            a string representation of a CSS selectors.
	 * @return the parsed selectors as {@link SelectorList}.
	 * @throws IOException
	 *             if something went wrong during parsing.
	 */
	public static SelectorList getSelectors(String selectors) throws IOException {
		try (StringReader sr = new StringReader(selectors)) {
			return PARSER.parseSelectors(new InputSource(sr));
		}
	}

	/**
	 * Gets a subclass of {@link CSSRule} from the given CSS kind.
	 *
	 * @param rule
	 *            the rule to cast.
	 * @param kind
	 *            the kind of rule to get.
	 * @return the cast rule or <code>null</code> if given rule is not of the given kind.
	 *
	 * @see CSSRule#getType()
	 */
	public static <T extends CSSRule> T getRule(CSSRule rule, int kind) {
		if (rule != null && rule.getType() == kind) {
			return (T) rule;
		}

		return null;
	}
}
