/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.description;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ServiceLoader;

/**
 * The class {@code StyleDescriptionFactory} allows to get all declared CSS descriptions.
 *
 * @see StyleDescription
 */
public class StyleDescriptionFactory {

	private static final List<StyleDescription> DESCRIPTIONS;

	static {
		DESCRIPTIONS = new LinkedList<StyleDescription>();
		ServiceLoader<StyleDescriptionPackage> serviceLoader = ServiceLoader.load(StyleDescriptionPackage.class);
		for (Iterator<StyleDescriptionPackage> iterator = serviceLoader.iterator(); iterator.hasNext();) {
			StyleDescriptionPackage descriptionPackage = iterator.next();
			descriptionPackage.loadDescriptions();
		}
	}

	private StyleDescriptionFactory() {
		// factory class, hide constructor
	}

	/**
	 * Adds a new CSS description to the factory.
	 *
	 * @param description
	 *            the description to add.
	 */
	public static final void addDescription(StyleDescription description) {
		DESCRIPTIONS.add(description);
	}

	/**
	 * Clears all the CSS descriptions.
	 */
	public static final void clearDescriptions() {
		DESCRIPTIONS.clear();
	}

	/**
	 * Gets an immutable list of available CSS descriptions.
	 *
	 * @return an immutable list of available CSS descriptions.
	 */
	public static final List<StyleDescription> getDescriptions() {
		return Collections.unmodifiableList(DESCRIPTIONS);
	}

}
