/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.jsgenerator.global;

import com.is2t.giml.jsgenerator.JavaScriptSourceToMicroEJSourceGenerator;
import com.is2t.java.nodes.Expression;
import com.is2t.java.nodes.LocalDeclaration;
import com.is2t.java.nodes.Statement;
import com.is2t.java.nodes.expressions.Assignment;
import com.is2t.java.nodes.expressions.LocalAccess;
import com.is2t.java.nodes.expressions.ObjectAllocation;

import jdk.nashorn.internal.ir.FunctionNode;

/**
 * Accessing a function as an object
 */
public class GlobalObjectFunction extends GlobalObjectUserDefined {

	private final FunctionNode node;

	public GlobalObjectFunction(String name, FunctionNode node) {
		super(name);
		this.node = node;
	}

	@Override
	public Statement generateInitialization(JavaScriptSourceToMicroEJSourceGenerator gen) {
		return new Assignment(new LocalAccess(new LocalDeclaration(gen.TYPE_OBJECT, gen.javaName(name), null)),
				new ObjectAllocation(gen.getTypeReference("GIMLFunctionClass" + name), new Expression[0]));

	}

	/**
	 * Gets the node.
	 *
	 * @return the node.
	 */
	public FunctionNode getNode() {
		return node;
	}

}
