/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.parser.test.xmlparser;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.hamcrest.CoreMatchers;
import org.junit.Assert;
import org.junit.Test;

import com.is2t.giml.xml.parser.XMLParser;
import com.is2t.giml.xml.parser.XMLParserOptions;
import com.is2t.giml.xml.parser.test.AbstractTest;

/**
 * Unit tests for {@link XMLParser} class.
 */
public class XMLParserTest extends AbstractTest {

	private static final String OTHER_EXTENSION = "other";

	@Test
	public void testDefaultXMLParserCreation() throws IOException {
		File tempFile = createTempFile("temp-file");
		XMLParser xmlParser = new XMLParser(tempFile);

		Assert.assertEquals(xmlParser.getOptions(), new XMLParserOptions());
	}

	@Test
	public void testChangeOptions() throws IOException {
		File tempFile = createTempFile("temp-file");
		XMLParser xmlParser = new XMLParser(tempFile);

		// Directly change an option
		XMLParserOptions originalOptions = xmlParser.getOptions();
		originalOptions.setXMLFileExtension(OTHER_EXTENSION);
		Assert.assertThat(originalOptions, CoreMatchers.not(CoreMatchers.equalTo(new XMLParserOptions())));

		// Set a new option object, equals options but not same instance
		XMLParserOptions otherOptions = new XMLParserOptions();
		otherOptions.setXMLFileExtension(OTHER_EXTENSION);
		xmlParser.setOptions(otherOptions);
		Assert.assertEquals(xmlParser.getOptions(), otherOptions);
		Assert.assertEquals(xmlParser.getOptions(), originalOptions);
		Assert.assertThat(xmlParser.getOptions(), CoreMatchers.not(CoreMatchers.sameInstance(originalOptions)));
	}

	@Test(expected = NullPointerException.class)
	public void testNullRootFile() throws FileNotFoundException {
		new XMLParser(null);
	}

	@Test(expected = FileNotFoundException.class)
	public void testInvalidRootFile() throws FileNotFoundException {
		new XMLParser(new File("/does/not/exist.ext"));
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullWorkingDir() throws IOException {
		File tempFile = createTempFile("temp-file");
		new XMLParser(tempFile, null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidWorkingDir() throws IOException {
		File tempFile = createTempFile("temp-file");
		List<File> dirs = new ArrayList<File>();
		dirs.add(new File("/does/not/exist/"));
		new XMLParser(tempFile, dirs);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullOptions() throws IOException {
		File tempFile = createTempFile("temp-file");
		new XMLParser(tempFile).setOptions(null);
	}

}
