package com.is2t.java.nodes;


/**
 * 
 * Java
 *
 * Includes interfaces and classes (see subclasses)
 * No Array or base type -> see TypeReference
 * 
 * Copyright 2008 IS2T. All rights reserved
 * IS2T PROPRIETARY/ Use is subject to license terms.
 */
public abstract class TypeDeclaration extends Statement {

	public Modifiers modifiers;
	
	public char[][] fullyQualifiedName;

	/**
	 * May be null when not extending/implementing super interfaces
	 */
	public TypeReference[] interfaces;	//not null. Not a TypeDefinition because the superType can be a pre-existing type

	public MethodDeclaration[] methods;	// not null
	
	public Statement[] fieldsOrInitializers;	//not null (element are instance of FieldDeclaration or Initializer)
	public TypeDeclaration[] members;	// not null
	
	public Comment[] comments;
	
	public TypeDeclaration(Modifiers modifiers, char[][] fullyQualifiedName, TypeReference[] interfaces, MethodDeclaration[] methods, Statement[] fieldsOrInitializers){
		this( fullyQualifiedName);
		initialize(modifiers, interfaces, methods, fieldsOrInitializers);
	}
	
	public TypeDeclaration (char[][] fullyQualifiedName){
		this.fullyQualifiedName = fullyQualifiedName;
	}
	
	public void initialize(Modifiers modifiers, TypeReference[] interfaces, MethodDeclaration[] methods, Statement[] fieldsOrInitializer) {
		this.modifiers = modifiers;
		this.interfaces = interfaces;
		this.methods = methods;
		this.fieldsOrInitializers = fieldsOrInitializer;
	}

		
	public char[] simpleName(){
		return fullyQualifiedName[fullyQualifiedName.length-1];
	}
	
	public void setComments(Comment[] comments){
		this.comments = comments;
	}
	
	public boolean isClass() {
		return false;
	}
	
	public boolean isInterface() {
		return false;
	}
	
	public AbstractMethodDeclaration lookForMethodAt( char[] name, TypeReference[] argTypes) {
		MethodDeclaration[] methods = this.methods;
		for (int i = methods.length; --i >= 0;) {
			if ( methods[i].equals( name, argTypes))
					return  methods[i];
		}
		return null;
	}

}
