package com.is2t.java.nodes.expressions;

import com.is2t.java.JavaSourceGenerator;
import com.is2t.java.nodes.AbstractMethodDeclaration;
import com.is2t.java.nodes.Expression;
import com.is2t.java.nodes.TypeReference;

public class MessageSend extends Expression {

	public static Expression[] NoArguments = new Expression[0];
	
	// if messageSend on a not a generated method (methods of pre-existing type for example) the MethodDeclaration is null
	// but the declarationTypeName is set.
	public AbstractMethodDeclaration declaration;	// not null if the method is a generated method
	public char[][] declarationTypeName;			// can be null. Declaration type name (useful for static method but null for current class static call)
	public TypeReference compileReceiverType;		// can be null. When the compileReceiverType is known 
	public char[] name;								// name of the method to call 
	
	public Expression receiver;						//  May be null when method is static
	
	public Expression[] arguments;					// never null
	
	public MessageSend(AbstractMethodDeclaration declaration, Expression receiver, Expression[] arguments){
		this.declaration = declaration;
		this.receiver = receiver;
		this.arguments = arguments;
		this.name = declaration.name;
		this.declarationTypeName= declaration.type.fullyQualifiedName;
	}
	
	public MessageSend(char[] preExistingMethodName, char[][] declarationTypeName, Expression receiver, Expression[] arguments){
		//this.declaration = null; vm done
		this.receiver = receiver;
		this.arguments = arguments;
		this.name = preExistingMethodName;
		this.declarationTypeName = declarationTypeName;
	}
	
	public MessageSend(char[] preExistingMethodName, char[][] declarationTypeName, Expression receiver){
		this(preExistingMethodName, declarationTypeName, receiver, NoArguments);
	}
	
	public MessageSend(char[] preExistingMethodName, TypeReference type, Expression receiver, Expression[] arguments){
		this( preExistingMethodName, type.fullyQualifiedName, receiver, arguments);
		compileReceiverType = type;
	}
	
	public MessageSend(char[] preExistingMethodName, TypeReference type, Expression receiver){
		this(preExistingMethodName, type, receiver, NoArguments);
	}
	
	
	public void generateExpressionUsing(JavaSourceGenerator gen) {
		gen.generateMessageSend(this);
	}
	
	/*
	 * Use it only when nodes are totally initialized
	 */
	public boolean isPreExistingMethod() {
		return declaration == null;
	}

}
