/*
 * Java
 * 
 * Copyright 2013 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.linker.map;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * <p>
 * Extends the legacy Map File interpretor behavior by specializing some features such as Listeners, launch, etc.
 * </p>
 */
public interface IMapFileInterpretor2 extends IMapFileInterpretor {

	/**
	 * <p>
	 * Initializes the interpretor configuration.
	 * </p>
	 * 
	 * @param mapFile the map file to handle (must exists).
	 * @param inputStream the input stream to be used for command retrieval/
	 * @param outputStream the output stream to be used for command output.
	 * @throws FileNotFoundException if the given map file doesn't exists
	 * @see #initialize(String, InputStream, OutputStream)
	 */
	void initialize(File mapFile, InputStream inputStream, OutputStream outputStream) throws FileNotFoundException;

	/**
	 * <p>
	 * Launches the map file interpretor and process the map file given at initialization time.
	 * </p>
	 * 
	 * @throws IllegalStateException if the interpretor has not been initialized.
	 * @throws InvalidMapFile if the map file loading fails.
	 * @see {@link #initialize(File, InputStream, OutputStream)}
	 * @see #initialize(String, InputStream, OutputStream)
	 */
	void launch() throws InvalidMapFile;

	void addListener(IMapFileInterpretorListener2 listener);

	void removeListener(IMapFileInterpretorListener2 listener);

	/**
	 * <p>
	 * Indicates if the interpretor is ready to work or not.
	 * </p>
	 * 
	 * @return <code>true</code> if the interpretor is ready to work, <code>false</code> otherwise
	 */
	boolean isReady();
}
