/*
 * Java
 *
 * Copyright 2009-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.is2t.microbsp.microui.geg;

import ej.microui.MicroUIPump;
import ej.microui.event.EventHandler;
import ej.microui.event.generator.Buttons;

/**
 * Instance created by SystemMicroUIGenerator
 */
public class SystemButtons extends Buttons {

	/**
	 * Driver MicroEJ specific: we use the same values as MicroUI event!
	 */
	private static final int DRIVER_ACTION_MASK = 0x0000ff00;
	private static final int DRIVER_ACTION_SHIFT = 8;
	private static final int DRIVER_BUTTON_MASK = 0x000000ff;
	private static final int DRIVER_BUTTON_SHIFT = 0;
	private static final int DRIVER_PRESS = 0x00;
	private static final int DRIVER_RELEASE = 0x01;
	private static final int DRIVER_LONG = 0x02;
	private static final int DRIVER_REPEAT = 0x03;

	/**
	 * @param nbButtons
	 *            available number of extended buttons
	 */
	public SystemButtons(int nbButtons) {
		super(nbButtons);
	}

	@Override
	public void convert(MicroUIPump pump, int event, EventHandler eventHandler) {
		int cmd = (event & DRIVER_ACTION_MASK) >>> DRIVER_ACTION_SHIFT;

		switch (cmd) {
		case DRIVER_PRESS:
			cmd = PRESSED;
			break;
		case DRIVER_RELEASE:
			cmd = RELEASED;
			break;
		case DRIVER_LONG:
			cmd = LONG;
			break;
		case DRIVER_REPEAT:
			cmd = REPEATED;
			break;
		default: {
			assert (false);
			return; // wrong driver ...
		}
		}

		int id = (event & DRIVER_BUTTON_MASK) >>> DRIVER_BUTTON_SHIFT;
		send(cmd, id, eventHandler);
	}

}
