/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2016-2018, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.is2t.support.security.x509;

import java.io.IOException;
import java.security.PublicKey;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;

import com.is2t.support.security.NativeKey;
import com.is2t.support.security.natives.X509CertSupportNatives;
import com.is2t.support.security.x509.X509CertImpl.EncodedFormat;

public class X509CertSupport {

	/*
	 * Parse the X509 certificate and returns the certificate encoded format
	 */
	public static EncodedFormat parse(X509CertImpl cert) throws CertificateException, IOException {
		byte[] data = cert.getEncoded();
		int ret = X509CertSupportNatives.parse(data, 0, data.length);

		if (ret < 0) {
			throw new IOException("Unable to parse certificate");
		}
		switch (ret) {
		case 0:
			return EncodedFormat.PEM;
		case 1:
			return EncodedFormat.DER;
		default:
			throw new IOException(
					"Unsupported certificate format. Only DER and PEM enconded format are supported.");
		}
	}

	public static void verify(X509CertImpl cert, PublicKey key) throws CertificateEncodingException {
		byte[] data = cert.getEncoded();
		byte[] keyBytes = ((NativeKey) key).getData();
		X509CertSupportNatives.verify(data, data.length, keyBytes, keyBytes.length);
	}
}
