/*
 * Java
 *
 * Copyright 2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.security.util;

import java.util.List;

/**
 *
 */
public class Arrays {

	/**
	 * Returns <tt>true</tt> if the two specified arrays of bytes are <i>equal</i> to one another. Two arrays are
	 * considered equal if both arrays contain the same number of elements, and all corresponding pairs of elements in
	 * the two arrays are equal. In other words, two arrays are equal if they contain the same elements in the same
	 * order. Also, two array references are considered equal if both are <tt>null</tt>.
	 * <p>
	 *
	 * @param a
	 *            one array to be tested for equality
	 * @param a2
	 *            the other array to be tested for equality
	 * @return <tt>true</tt> if the two arrays are equal
	 */
	public static boolean equals(byte[] a, byte[] a2) {
		if (a == a2) {
			return true;
		}
		if (a == null || a2 == null) {
			return false;
		}

		int length = a.length;
		if (a2.length != length) {
			return false;
		}

		for (int i = 0; i < length; i++) {
			if (a[i] != a2[i]) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Returns a hash code based on the contents of the specified array. For any two <tt>byte</tt> arrays <tt>a</tt> and
	 * <tt>b</tt> such that <tt>Arrays.equals(a, b)</tt>, it is also the case that
	 * <tt>Arrays.hashCode(a) == Arrays.hashCode(b)</tt>.
	 *
	 * <p>
	 * The value returned by this method is the same value that would be obtained by invoking the {@link List#hashCode()
	 * <tt>hashCode</tt>} method on a {@link List} containing a sequence of {@link Byte} instances representing the
	 * elements of <tt>a</tt> in the same order. If <tt>a</tt> is <tt>null</tt>, this method returns 0.
	 *
	 * @param a
	 *            the array whose hash value to compute
	 * @return a content-based hash code for <tt>a</tt>
	 * @since 1.5
	 */
	public static int hashCode(byte a[]) {
		if (a == null) {
			return 0;
		}

		int result = 1;
		for (byte element : a) {
			result = 31 * result + element;
		}

		return result;
	}

	/**
	 * Assigns the specified byte value to each element of the specified array of
	 * bytes.
	 *
	 * @param a
	 *            the array to be filled
	 * @param val
	 *            the value to be stored in all elements of the array
	 */
	public static void fill(byte[] a, byte val) {
		for (int i = 0, len = a.length; i < len; i++) {
			a[i] = val;
		}
	}
}
