/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.vivante.utils;

import com.verisilicon.vivante.vg_lite;
import com.verisilicon.vivante.vg_lite_gradient;
import com.verisilicon.vivante.vg_lite_matrix;
import com.verisilicon.vivante.vg_lite_path;

import ej.microui.display.GraphicsContext;

/**
 * Represents a path with a gradient filling.
 */
public class GradientPath implements Path {
	private final vg_lite_path path;
	private final vg_lite_gradient gradient;
	private final int fillRule;

	/**
	 * Creates a colored path.
	 *
	 * @param path
	 *            the path.
	 * @param gradient
	 *            the gradient.
	 * @param fillRule
	 *            the fill rule (one of {@link vg_lite#VG_LITE_FILL_EVEN_ODD} and
	 *            {@link vg_lite#VG_LITE_FILL_NON_ZERO}).
	 */
	public GradientPath(vg_lite_path path, vg_lite_gradient gradient, int fillRule) {
		this.path = path;
		this.gradient = gradient;
		this.fillRule = fillRule;
	}

	@Override
	public vg_lite_path getVGLitePath() {
		return this.path;
	}

	/**
	 * Gets the VGLite gradient of the path.
	 *
	 * @return the VGLite gradient.
	 */
	public vg_lite_gradient getGradient() {
		return this.gradient;
	}

	@Override
	public void draw(GraphicsContext g, vg_lite_matrix matrix) {
		vg_lite.vg_lite_draw_gradient(g, this.path, this.fillRule, matrix, this.gradient,
				vg_lite.VG_LITE_BLEND_SRC_OVER);
	}
}
