/*
 * Java
 *
 * Copyright 2019-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.verisilicon.vivante;

/**
 *
 */
public class vg_lite_gradient implements AutoCloseable {
	private final int handle;

	/**
	 * Create a vg_lite gradient object
	 */
	public vg_lite_gradient() {
		this.handle = Nvg_lite_gradient.create();

		if (this.handle == 0) {
			throw new RuntimeException();
		}
	}

	@Override
	public void close() {
		Nvg_lite_gradient.destroy(this.handle);
	}

	/**
	 * Gets the handle of a gradient
	 *
	 * @return The handle
	 */
	public int getHandle() {
		return this.handle;
	}

	/**
	 * Gets the matrix of the gradient
	 *
	 * @return The handle
	 */
	public vg_lite_matrix getMatrix() {
		return new vg_lite_matrix(Nvg_lite_gradient.get_matrix_handle(this.handle));
	}

	/**
	 * See <code>vg_lite_init_grad</code> C function.
	 */
	public static void vg_lite_init_grad(vg_lite_gradient gradient) {
		int ret = Nvg_lite.vg_lite_init_grad(gradient.getHandle());

		if (0 != ret) {
			throw new vg_lite_error(ret);
		}
	}

	/**
	 * See <code>vg_lite_set_grad</code> C function.
	 */
	public static void vg_lite_set_grad(vg_lite_gradient gradient, int count, int[] colors, int[] stops) {
		int ret = Nvg_lite.vg_lite_set_grad(gradient.getHandle(), count, colors, stops);

		if (0 != ret) {
			throw new vg_lite_error(ret);
		}
	}

	public static void vg_lite_update_grad(vg_lite_gradient gradient) {
		int ret = Nvg_lite.vg_lite_update_grad(gradient.getHandle());

		if (0 != ret) {
			throw new vg_lite_error(ret);
		}

	}
}
