/*
 * Java
 *
 * Copyright 2019-2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.verisilicon.vivante;

/**
 * Class describing a vg_lite_path
 *
 * Expected to be an immutable so that it can be passed as-is to native side
 */
public class vg_lite_path {

	/**
	 * High quality 16x anti-aliasing path.
	 * <p>
	 * Each path should specify a quality hint for the hardware. The path generation tool will generate the quality hint
	 * based on the complexity of the path.
	 */
	public static final int VG_LITE_HIGH = 0;

	/**
	 * Upper quality 8x anti-aliasing path.
	 *
	 * @see #VG_LITE_HIGH
	 */
	public static final int VG_LITE_UPPER = 1;

	/**
	 * Medium quality 4x anti-aliasing path.
	 *
	 * @see #VG_LITE_HIGH
	 */
	public static final int VG_LITE_MEDIUM = 2;

	/**
	 * Low quality pat without any anti-aliasing.
	 *
	 * @see #VG_LITE_HIGH
	 */
	public static final int VG_LITE_LOW = 3;

	/**
	 * Signed 8-bit coordinates.
	 * <p>
	 * Each path can have a separate coordinate system. The path generation tool will find the most optimal coordinate
	 * system for any given path based on its dimensions and input coordinates.
	 */
	public static final int VG_LITE_S8 = 0;

	/**
	 * Signed 16-bit coordinates.
	 *
	 * @see #VG_LITE_S8
	 */
	public static final int VG_LITE_S16 = 1;

	/**
	 * Signed 32-bit coordinates.
	 *
	 * @see #VG_LITE_S8
	 */
	public static final int VG_LITE_S32 = 2;

	/**
	 * 32-bit floating point coordinates.
	 *
	 * @see #VG_LITE_S8
	 */
	public static final int VG_LITE_FP32 = 3;

	/**
	 * Command to close a path. No command data.
	 * <p>
	 * A path is defined by including some command elements on which the property specifies the command data. The
	 * available number of command data depends on the command. A path is always terminated by the
	 * {@link #VG_LITE_CMD_CLOSE} command.
	 */
	public static final int VG_LITE_CMD_CLOSE = 0;

	/**
	 * Command to move to an absolute destination. Command data is the destination absolute point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_MOVE = 2;

	/**
	 * Command to move to a relative destination. Command data is the destination relative point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_MOVE_REL = 3;

	/**
	 * Command to define a line from current position to an absolute destination. Command data is the line absolute
	 * end-point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_LINE = 4;

	/**
	 * Command to define a line from current position to a relative destination. Command data is the line relative
	 * end-point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_LINE_REL = 5;

	/**
	 * Quadratic curve from current position to an absolute destination using one control point. Command data is the
	 * control point (cx, cy) and the curve absolute end-point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_QUAD = 6;

	/**
	 * Quadratic curve from current position to an absolute destination using one control point. Command data is the
	 * control point (cx, cy) and the curve relative end-point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_QUAD_REL = 7;

	/**
	 * Cubic curve from current position to an absolute destination using two control points. Command data is the
	 * control points (cx1, cy1, cx2, cy2) and the curve absolute end-point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_CUBIC = 8;

	/**
	 * Cubic curve from current position to an absolute destination using two control points. Command data is the
	 * control points (cx1, cy1, cx2, cy2) and the curve relative end-point (x, y).
	 *
	 * @see #VG_LITE_CMD_CLOSE
	 */
	public static final int VG_LITE_CMD_CUBIC_REL = 9;

	/**
	 * Creates a new instance of a vg_lite_path.
	 *
	 * When used with this constructor the path is not usable. It must be updated with <code>vg_lite_path.update</code>
	 */
	public vg_lite_path() {
		throw new RuntimeException();
	}

	/**
	 * Creates a new instance of a vg_lite_path.
	 *
	 * @param left
	 *            left bound of the path
	 * @param top
	 *            top bound of the path
	 * @param right
	 *            right bound of the path
	 * @param bottom
	 *            bottom bound of the path
	 * @param quality
	 *            quality of the path (see vg_lite.h for definition)
	 * @param format
	 *            format of the path (see vg_lite.h for definition)
	 * @param path
	 *            byte array containing the actual path data
	 * @param offset
	 *            offset in the path array
	 * @param length
	 *            length in the path array
	 */
	public vg_lite_path(float left, float top, float right, float bottom, int quality, int format, byte[] path,
			int offset, int length) {
		throw new RuntimeException();
	}

	/**
	 * Creates a new instance of a vg_lite_path.
	 *
	 * @param left
	 *            left bound of the path
	 * @param top
	 *            top bound of the path
	 * @param right
	 *            right bound of the path
	 * @param bottom
	 *            bottom bound of the path
	 * @param quality
	 *            quality of the path (see vg_lite.h for definition)
	 * @param format
	 *            format of the path (see vg_lite.h for definition)
	 * @param path
	 *            byte array containing the actual path data
	 */
	public vg_lite_path(float left, float top, float right, float bottom, int quality, int format, byte[] path) {
		throw new RuntimeException();
	}

	/**
	 * Updates a vg_lite_path.
	 *
	 * @param left
	 *            left bound of the path
	 * @param top
	 *            top bound of the path
	 * @param right
	 *            right bound of the path
	 * @param bottom
	 *            bottom bound of the path
	 * @param quality
	 *            quality of the path (see vg_lite.h for definition)
	 * @param format
	 *            format of the path (see vg_lite.h for definition)
	 * @param path
	 *            byte array containing the actual path data
	 * @param offset
	 *            offset in the path array
	 * @param length
	 *            length in the path array
	 */
	public void update(float left, float top, float right, float bottom, int quality, int format, byte[] path,
			int offset, int length) {
		throw new RuntimeException();
	}

	/**
	 * Updates a vg_lite_path.
	 *
	 * @param left
	 *            left bound of the path
	 * @param top
	 *            top bound of the path
	 * @param right
	 *            right bound of the path
	 * @param bottom
	 *            bottom bound of the path
	 * @param quality
	 *            quality of the path (see vg_lite.h for definition)
	 * @param format
	 *            format of the path (see vg_lite.h for definition)
	 * @param path
	 *            byte array containing the actual path data
	 */
	public void update(float left, float top, float right, float bottom, int quality, int format, byte[] path) {
		throw new RuntimeException();
	}

	/**
	 * @return the path quality (see vg_lite.h for definition)
	 */
	public int getQuality() {
		throw new RuntimeException();
	}

	/**
	 * @return the path format (see vg_lite.h for definition)
	 */
	public int getFormat() {
		throw new RuntimeException();
	}

	/**
	 * @return a byte array containing the path data
	 */
	public byte[] getPath() {
		throw new RuntimeException();
	}

	/**
	 * @return the offset in the byte array containing the path data
	 */
	public int getPathOffset() {
		throw new RuntimeException();
	}

	/**
	 * @return the path length of the path
	 */
	public int getPathLength() {
		throw new RuntimeException();
	}

	/**
	 * @return the left bound of the path
	 */
	public float getLeft() {
		throw new RuntimeException();
	}

	/**
	 * @return the right bound of the path
	 */
	public float getRight() {
		throw new RuntimeException();
	}

	/**
	 * @return the top bound of the path
	 */
	public float getTop() {
		throw new RuntimeException();
	}

	/**
	 * @return the bottom bound of the path
	 */
	public float getBottom() {
		throw new RuntimeException();
	}
}
